@@ -1,8 +1,8 @@
 from django.db.models import options
 
-__version__ = '0.7.12'
+__version__ = '0.8.0'
 options.DEFAULT_NAMES += (
     'lookup_field', 'rdf_type', 'rdf_context', 'auto_author', 'auto_author_field', 'owner_field', 'view_set',
-    'container_path', 'permission_classes', 'serializer_fields', 'nested_fields', 'depth', 'anonymous_perms',
-    'authenticated_perms', 'owner_perms')
+    'container_path', 'permission_classes', 'serializer_fields', 'nested_fields', 'nested_fields_exclude', 'depth',
+    'anonymous_perms', 'authenticated_perms', 'owner_perms')
 default_app_config = 'djangoldp.apps.DjangoldpConfig'
