@@ -12,6 +12,7 @@ from django.dispatch import receiver
 from django.urls import reverse_lazy, get_resolver
 from django.utils.datastructures import MultiValueDictKeyError
 from django.utils.decorators import classonlymethod
+from rest_framework.utils import model_meta
 
 from djangoldp.fields import LDPUrlField
 from djangoldp.permissions import LDPPermissions
@@ -22,12 +23,28 @@ logger = logging.getLogger('djangoldp')
 
 
 class LDPModelManager(models.Manager):
-    # an alternative to all() which exlcudes external resources
     def local(self):
+        '''an alternative to all() which exlcudes external resources'''
         queryset = super(LDPModelManager, self).all()
         internal_ids = [x.pk for x in queryset if not Model.is_external(x)]
         return queryset.filter(pk__in=internal_ids)
 
+    def nested_fields(self):
+        '''parses the relations on the model, and returns a list of nested field names'''
+        nested_fields = set()
+        # include all many-to-many relations
+        for field_name, relation_info in model_meta.get_field_info(self.model).relations.items():
+            if relation_info.to_many:
+                nested_fields.add(field_name)
+        # include all nested fields explicitly included on the model
+        nested_fields.update(set(Model.get_meta(self.model, 'nested_fields', set())))
+        # exclude anything marked explicitly to be excluded
+        nested_fields = nested_fields.difference(set(Model.get_meta(self.model, 'nested_fields_exclude', set())))
+        return list(nested_fields)
+
+    def fields(self):
+        return self.nested_fields()
+
 
 class Model(models.Model):
     urlid = LDPUrlField(blank=True, null=True, unique=True)
@@ -35,6 +52,7 @@ class Model(models.Model):
     allow_create_backlink = models.BooleanField(default=True,
                                                 help_text='set to False to disable backlink creation after Model save')
     objects = LDPModelManager()
+    nested = LDPModelManager()
 
     def __init__(self, *args, **kwargs):
         super(Model, self).__init__(*args, **kwargs)
@@ -192,6 +210,16 @@ class Model(models.Model):
                 field_tuples['username'] = str(uuid.uuid4())
             return model.objects.create(urlid=urlid, is_backlink=True, **field_tuples)
 
+    @classonlymethod
+    def get_or_create_external(cls, model, urlid, **kwargs):
+        '''
+        checks that the parameterised urlid is external and then returns the result of Model.get_or_create
+        :raises ObjectDoesNotExist: if the urlid is not external and the object doesn't exist
+        '''
+        if not Model.is_external(urlid) and not model.objects.filter(urlid=urlid).exists():
+            raise ObjectDoesNotExist
+        return Model.get_or_create(model, urlid, **kwargs)
+
     @classonlymethod
     def get_model_rdf_type(cls, model):
         if model is get_user_model():
@@ -234,9 +262,15 @@ class Model(models.Model):
 
     @classmethod
     def is_external(cls, value):
-        '''returns True if the urlid of the value passed is from an external source'''
+        '''
+        :param value: string urlid or an instance with urlid field
+        :return: True if the urlid is external to the server, False otherwise
+        '''
         try:
-            return value.urlid is not None and not value.urlid.startswith(settings.SITE_URL)
+            if not isinstance(value, str):
+                value = value.urlid
+
+            return value is not None and not value.startswith(settings.SITE_URL)
         except:
             return False
 
@@ -273,8 +307,9 @@ class Activity(Model):
 
 class Follower(Model):
     '''Models a subscription on a model. When the model is saved, an Update activity will be sent to the inbox'''
-    object = models.URLField()
-    inbox = models.URLField()
+    object = models.URLField(help_text='the object being followed')
+    inbox = models.URLField(help_text='the inbox recipient of updates')
+    follower = models.URLField(help_text='(optional) the resource/actor following the object', blank=True)
 
     def __str__(self):
         return 'Inbox ' + str(self.inbox) + ' on ' + str(self.object)
