@@ -320,7 +320,7 @@ class LDPSerializer(HyperlinkedModelSerializer):
                                                                                         'permission_classes',
                                                                                         [LDPPermissions]),
                                                       fields=Model.get_meta(model_class, 'serializer_fields', []),
-                                                      nested_fields=Model.get_meta(model_class, 'nested_fields', []))
+                                                      nested_fields=model_class.nested.nested_fields())
                     parent_depth = max(getattr(self.parent.Meta, "depth", 0) - 1, 0)
                     serializer_generator.depth = parent_depth
                     serializer = serializer_generator.build_read_serializer()(context=self.parent.context)
@@ -627,7 +627,7 @@ class LDPSerializer(HyperlinkedModelSerializer):
                         model, sub_inst = Model.resolve(field_dict['urlid'])
                 # remote resource - get backlinked copy
                 elif hasattr(field_model, 'urlid'):
-                    sub_inst = Model.get_or_create(field_model, field_dict['urlid'])
+                    sub_inst = Model.get_or_create_external(field_model, field_dict['urlid'])
             # try slug field, assuming that this is a local resource
             elif slug_field in field_dict:
                 kwargs = {slug_field: field_dict[slug_field]}
